/**
 * @file 	LMP.java
 * 
 * @date 	Jan 15, 2005
 * @author 	Alexandre Bouchard
 */

package ca.mcgill.lmp;

/**
 *
 */
public class LMP
{
    private MeasurableSet stateSpace;
    
    private TransitionKernel [] kernels;
    
    private State currentState;
    
    public LMP(MeasurableSet stateSpace, TransitionKernel [] kernels, State startState)
    {
        this.stateSpace = stateSpace;
        this.kernels = kernels;
        initState(startState);
    }
    
    public void initState(State startState)
    {
        currentState = startState;
    }
    
    public State getCurrentState()
    {
        return currentState;
    }
    
    public MeasurableSet getStateSpace()
    {
        return stateSpace;
    }
    
    public int getNumberOfLabels()
    {
        return kernels.length;
    }
    
    public TransitionKernel getTransitionKernel(int index)
    {
        if (index < 0 || index >= getNumberOfLabels())
        {
            throw new RuntimeException("LMP.getTransitionKernel(): invalid label index.");
        }
        return kernels[index];
    }
}
