/**
 * @file	Example1.java
 * 
 * @date	Mar 8, 2005
 * @author	Alexandre Bouchard
 */
package ca.mcgill.lmp.examples;

import ca.mcgill.lmp.LMP;
import ca.mcgill.lmp.MeasurableFunction;
import ca.mcgill.lmp.State;
import ca.mcgill.lmp.approx.LMPApproximator;
import ca.mcgill.lmp.util.FunctionsFamily;
import ca.mcgill.lmp.util.MultiInterval;
import ca.mcgill.lmp.util.RnState;
import ca.mcgill.lmp.util.kernels.DensityKernel;
import ca.mcgill.lmp.util.measures.BorelMeasure;

/**
 * 
 *
 */
public class Example1
{

    public static void main(String[] args)
    {        
        MultiInterval stateSpace = new MultiInterval(0, 1);
        BorelMeasure canonicalMeasure = new BorelMeasure(1);
        DensityKernel [] kernels = new DensityKernel[2];
        kernels[0] = new DensityKernel(new SwimFamily(), canonicalMeasure);
        kernels[1] = new DensityKernel(new JumpFamily(), canonicalMeasure);
        LMP lmp = new LMP(stateSpace, kernels, canonicalMeasure.sampleIn());
        
        LMPApproximator approximator = new LMPApproximator(lmp, canonicalMeasure);
        LMP approximation = approximator.approximate(0.5, 3);
        
        System.out.println("Test complete!");
        System.out.println(approximation.toString());
        System.out.println(approximation.getTransitionKernel(0).toString());
        System.out.println(approximation.getTransitionKernel(1).toString());
    }
}

class SwimFamily implements FunctionsFamily
{
    private MultiInterval zeroOneHalf = new MultiInterval(0, 0.5);

    /**
     * @param index
     * @return
     */
    public MeasurableFunction getFunction(State index)
    {
        return new SwimDensity(index);
    }
    
    public String toString()
    {
        return "swim";
    }
    
    class SwimDensity implements MeasurableFunction
    {
        private double sourceState;
        
        public SwimDensity(State source)
        {
            sourceState = ((RnState) source).getCoordinate(0);
        }

        /**
         * @param s
         * @return
         */
        public double evaluateAt(State s)
        {
            if (zeroOneHalf.contains(sourceState) && zeroOneHalf.contains(s))
            {
                return 2.0;
            }
            else
            {
                return 0.0;
            }
        }
        
    }
}

class JumpFamily implements FunctionsFamily
{
    private MultiInterval zeroOneHalf = new MultiInterval(0, 0.5);
    private MultiInterval oneHalfOne = new MultiInterval(0.5, 1, false, true);

    /**
     * @param index
     * @return
     */
    public MeasurableFunction getFunction(State index)
    {
        return new JumpDensity(index);
    }
    
    public String toString()
    {
        return "jump";
    }
    
    class JumpDensity implements MeasurableFunction
    {
        private double sourceState;
        private MultiInterval xOneHalf;
        
        public JumpDensity(State s)
        {
            sourceState = ((RnState) s).getCoordinate(0);
            xOneHalf = new MultiInterval(sourceState, 0.5);
        }

        /**
         * @param s
         * @return
         */
        public double evaluateAt(State destinationState)
        {
            if (zeroOneHalf.contains(sourceState) && xOneHalf.contains(destinationState))
            {
                return 2.0;
            }
            else if (zeroOneHalf.contains(sourceState) && oneHalfOne.contains(destinationState))
            {
                return 4*sourceState;
            }
            else
            {
                return 0.0;
            }
        }
        
    }
}