/**
 * @file	BorelMeasure.java
 * 
 * @date	Feb 3, 2005
 * @author	Alexandre Bouchard
 */
package ca.mcgill.lmp.util.measures;

import ca.mcgill.lmp.EmptySetException;
import ca.mcgill.lmp.MeasurableSet;
import ca.mcgill.lmp.Measure;
import ca.mcgill.lmp.State;
import ca.mcgill.lmp.approx.MonteCarlo;
import ca.mcgill.lmp.util.MultiInterval;
import ca.mcgill.lmp.util.functions.CharacteristicFunction;

/**
 * 
 *
 */
public class BorelMeasure implements Measure
{
    /**
     * Comment for <code>dimensionality</code>
     */
    private int dimensionality;
    
    private MultiInterval X;
    
    /**
     * @param dimensionality
     */
    public BorelMeasure(int dimensionality)
    {
        this.dimensionality = dimensionality;
        X = new MultiInterval(dimensionality);
    }

    /**
     * @param B
     * @return
     */
    public double measure(MeasurableSet B)
    {
        if (B instanceof MultiInterval)
        {
            return ((MultiInterval) B).area();
        }
        MonteCarlo integrator = new MonteCarlo(this);
        return integrator.integrate(new CharacteristicFunction(B));
    }
    
    public State sampleIn()
    {
        try
        {
            return sampleIn(X);
        }
        catch (Exception e)
        {
            System.out.println("" + e);
            return null;
        }
    }

    /**
     * @param B
     * @return
     */
    public State sampleIn(MeasurableSet B) throws EmptySetException
    {
        if (B instanceof MultiInterval)
        {
            MultiInterval interval = (MultiInterval) B;
            return interval.sampleUniformly();
        }
        State currentState = X.sampleUniformly();
        for (int i = 0; i < MonteCarlo.DEFAULT_NUMBER_OF_TRIALS; i++)
        {
            if (B.contains(currentState))
            {
                return currentState;
            }
            currentState = X.sampleUniformly();
        }
        throw new EmptySetException();
    }

    /**
     * @return
     */
    public MeasurableSet getX()
    {
        return X;
    }
    
    public String toString()
    {
        return "Borel measure on [0, 1]^" + dimensionality;
    }

    /**
     * @return
     */
    public State sample()
    {
        try
        {
            return sampleIn(getX());
        }
        catch (EmptySetException ese)
        {
            System.out.println(ese.toString());
            return null;
        }
    }
}
